#!/usr/bin/env Rscript

#################################
# Functions for fitting ARIMA models
#
#   loadTrace - Load trace file
#
#   saveARIMA - Save ARIMA model to XML file
#
#   fitARIMA - Fit ARIMA model to trace
#
#################################

# Load a trace file
#   file: Path to trace file
loadTrace <- function(file) {
	tracedata <- scan(file=file, flush=TRUE)
	tracedata <- tracedata[2:length(tracedata)]
	tracedata
}

#################################

# Save an ARIMA model to a file
#   file: File to save the model
#   model: Model of class "Arima"
saveARIMA <- function(file, model) {
	# get model parameters
	# order of model (p, d, q)
	ARIMAorder <- c(0, 0, 0)
	# autoregressive coefficients
	ARcoef <- 0	
	# moving average coefficients
	MAcoef <- 0	
	# mean
	ARIMAmean <- 0
	# variance
	ARIMAvar <- 0
	if (class(model) == "Arima") {
		# get model parameters for class "Arima"
		ARIMAorder[1] <- model$arma[1]
		ARIMAorder[2] <- model$arma[6]
		ARIMAorder[3] <- model$arma[2]
		if (ARIMAorder[1] > 0) {
			ARcoef <- model$coef[1:ARIMAorder[1]]
		}
		if (ARIMAorder[3] > 0) {
			MAcoef <- model$coef[(ARIMAorder[1]+1):(ARIMAorder[1]+ARIMAorder[3])]
		}
		if (length(model$coef) > (ARIMAorder[1] + ARIMAorder[3])) {
			ARIMAmean <- model$coef[length(model$coef)]
			attributes(ARIMAmean) <- NULL
		}
		ARIMAvar <- model$sigma2
	} else {
		stop("Invalid model.")
	}
	if (sum(ARIMAorder) == 0) {
		stop("Invalid model order.")
	}
	# delete old contents of file
	cat("", file=file, append=FALSE)
	# write model parameters to file
	cat("<?xml version=\"1.0\"?>\n", file=file, append=TRUE)
	cat("<profido>\n", file=file, append=TRUE)
	cat("<arima>\n", file=file, append=TRUE)
	cat("<arcount>", ARIMAorder[1], "</arcount>\n", file=file, append=TRUE)
	cat("<ar>", ARcoef, "</ar>\n", file=file, append=TRUE)
	cat("<macount>", ARIMAorder[3], "</macount>\n", file=file, append=TRUE)
	cat("<ma>", MAcoef, "</ma>\n", file=file, append=TRUE)
	cat("<d>", ARIMAorder[2], "</d>\n", file=file, append=TRUE)
	cat("<mean>", ARIMAmean, "</mean>\n", file=file, append=TRUE)
	cat("<variance>", ARIMAvar, "</variance>\n", file=file, append=TRUE)
	cat("</arima>\n", file=file, append=TRUE)
	cat("</profido>\n", file=file, append=TRUE)
}

#################################

# Fit an ARIMA model to a trace.
#
# order: of the ARIMA model (p,d, q)
#    p: order of AR part
#    d: order of differencing
#    q: order of MA part
# trace: Vector containing the trace elements
#
# return: ARIMA model
#
fitARIMA <- function(order=c(2,0,0), trace) {
	# check if order is allowed
	if ((length(order) != 3) || (order[1]<0) || (order[2]<0) || (order[3]<0)) {
		stop("Order not allowed for ARIMA model.")
	}
	# fit ARIMA model to trace
	arimamodel <- arima(x=trace, order=order);
	# return model
	arimamodel
}

#################################

# read command line arguments
args <- commandArgs(TRUE)
if (length(args) < 5) {
	stop("Missing command line parameters.")
}
infile <- args[1];
outfile <- args[2];
arcount <- type.convert(args[3]);
d <- type.convert(args[4]);
macount <- type.convert(args[5]);
# load trace
trace <- loadTrace(infile);
# fit ARIMA model to trace
arimamodel <- fitARIMA(c(arcount, d, macount), trace);
# save ARIMA model to outfile in XML format
saveARIMA(outfile, arimamodel);
q()

#################################

