#!/usr/bin/env Rscript

#################################
# Functions for testing the hypothesis whether two
# traces are from the same distributions. The used
# tests are:
#   ks.test     two sample Kolmogorov-Smirnov test
#   t.test      two sample t-test 
#   chisq.test  two sample chi-squared test
# 
#
#   loadTrace      - Load trace file
#
#   saveRes        - Save test results in a LaTeX tabular environment
#
#   test*          - perform corresponding test
#
#################################

# Load a trace file
#   file: Path to trace file
loadTrace <- function(file) {
	tracedata <- scan(file=file, flush=TRUE)
	tracedata <- tracedata[2:length(tracedata)]
	tracedata
}

#################################
# Save test results to a file
#   file:       File to save the results in LaTeX format
#   resKS:      Test result of KS-Test
#   resT:       Test result of T-Test
#   resChisq:   Test result of ChiSquared-Test
#saveRes <- function(file, resKS, resT, resChisq) {
#        # check whether all test results are of the correct type
#        no_tests <- 3;
#	if (class(resKS)    == "htest") { no_tests <- no_tests - 1; } 
#	if (class(resT)     == "htest") { no_tests <- no_tests - 1; } 
#	if (class(resChisq) == "htest") { no_tests <- no_tests - 1; } 
#        if (no_tests == 0) {
#                # extract parameters to be inserted into the tabular environment
#		KSpval    <- resKS$p.value
#		Tpval     <- resT$p.value
#		Chisqpval <- resChisq$p.value
#	} else {
#		stop("Invalid test result types.")
#	}
#	# delete old contents of file
#	cat("", file=file, append=FALSE)
#	# write test results  to file
#	cat("\\begin{tabular}[t]{|r|r|}\n", file=file, append=TRUE)        
#	cat("\\hline\n", file=file, append=TRUE)        
#	cat("\\textbf{Test} & \\textbf{P-Value}\\\\ \n", file=file, append=TRUE)        
#	cat("\\hline\n", file=file, append=TRUE)        
#	cat("Kolmogorov-Smirnov Test& ", KSpval, "\\\\ \n", file=file, append=TRUE)        
#	cat("T Test & ", Tpval, "\\\\ \n", file=file, append=TRUE)        
#	cat("Pearson's Chi-squared Test & ", Chisqpval, "\\\\ \n", file=file, append=TRUE)        
#	cat("\\hline\n", file=file, append=TRUE)        
#	cat("\\end{tabular}\n", file=file, append=TRUE)        
#}
saveRes <- function(file, resKS, resChisq) {
        # check whether all test results are of the correct type
        no_tests <- 2;
	if (class(resKS)    == "htest") { no_tests <- no_tests - 1; } 
	if (class(resChisq) == "htest") { no_tests <- no_tests - 1; } 
        if (no_tests == 0) {
                # extract parameters to be inserted into the tabular environment
		KSpval    <- resKS$p.value
                KSDval    <- resKS$statistic
		Chisqpval <- resChisq$p.value
		Chisqstat <- resChisq$statistic
                
	} else {
		stop("Invalid test result types.")
	}
	# delete old contents of file
	cat("", file=file, append=FALSE)
	# write test results  to file
	cat("\\begin{tabular}[t]{|r|r|}\n", file=file, append=TRUE)        
	cat("\\hline\n", file=file, append=TRUE)        
	cat("\\textbf{Test} & \\textbf{P-Value}\\\\ \n", file=file, append=TRUE)        
	cat("\\hline\n", file=file, append=TRUE)        
	cat("Kolmogorov-Smirnov Test& ", KSpval, "\\\\ \n", file=file, append=TRUE)        
	cat("Pearson's Chi-squared Test & ", Chisqpval, "\\\\ \n", file=file, append=TRUE)        
	cat("\\hline\n", file=file, append=TRUE)        
	cat("\\end{tabular}\n", file=file, append=TRUE)        
}

#################################

# Perform KS-test
#
# trace1: Vector containing the elements of the first trace
# trace2: Vector containing the elements of the second trace
#
# return: test result
#
testKS <- function(trace1, trace2) {
	# perform test
	ksres    <-    ks.test(x=trace1, y=trace2);
	# return result
	ksres
}

#################################

# Perform T-test
#
# trace1: Vector containing the elements of the first trace
# trace2: Vector containing the elements of the second trace
#
# return: test result
#
testT <- function(trace1, trace2) {
	# perform test
	tres     <-     t.test(x=trace1, y=trace2);
	# return result
	tres
}


#################################

# Auxiliary function for Chi-Squared-test
#
# x,y: vectors with number of occurrences for an interval, i.e. x[i],y[i] is number of occurrences of interval i
# lower: lower bound for x[i] and y[i]
#
# In case of vec1[i] < lower the elements vec1[i+1], vec1[i+2], ... are added to vec1[i] until vec1[i]>=lower.
# The same is done for the other vector vec2, so that the same intervals are merged, even if the second vector
# does not violate vec2[i]>=lower. 
#
# return: z, a matrix which has as rows the number of occurrences of the changed x[i], y[i] whith 
#             x[i],y[i] >= lower
#
# Background: chisq.test needs a matrix with two columns, thus z is generated in this procedure.
#             Beforehand x and y are changed such that the interval count for x and y is at least >= lower
#             with a default value of 5 which is the recommended lower bound (N p_i >= 5) for the
#             number of occurrences within an interval for the Chi-Squared-test.
#
mergevec <- function(x,y,lower=5) {
  i <- 1;
  j <- i + 1;
  # in principal this works even for x,y of different length,
  # even though x and y should have the same length here, see testChisq
  len <- min(length(x),length(y));
  while (j <= len) {
    if ((x[i] < lower) || (y[i] < lower)) 
         { x[i] <- x[i] + x[j]; x[j] <- 0;
           y[i] <- y[i] + y[j]; y[j] <- 0; j <- j + 1; }
    else { i <- i+1; j <- i + 1; } 
  }
  xnew <- x[1:i];
  ynew <- y[1:i];
  z <- matrix(c(xnew,ynew),ncol=2);
  # return table for Chi-Squared-Test
  z
}

#################################

# Perform Chi-Squared-test
#
# trace1: Vector containing the elements of the first trace
# trace2: Vector containing the elements of the second trace
#
# return: test result
#
testChisq <- function(trace1, trace2) {
        # For the Chi-Squared test we must build up table of intervals counting the number of
        # occurrences within an interval.
        # We select sqrt(trace length) as the number of intervals and transform the 
        # traces to integer numbers in order to use R's tabulate function
        # The transformation is a linear mapping, mapping x \in [min(trace),max(trace)]
        # to x' \in [1,sqrtn] by x'=ax+b

        # We take the larger trace as a representation of the "theoretical" distribution,
        # since the Chi-Squared test is problematic for large N.
        if (length(trace1) < length(trace2)) {
           # swap trace1 and trace2
           tmp <- trace1; trace1 <- trace2; trace2 <- tmp;
        }

        sqrtn <- sqrt(length(trace2));
        a <- (sqrtn - 1)/(max(trace2)-min(trace2));
        b <- 1 - (a * min(trace2));        

        x <- tabulate(round(a*trace1+b),nbins=round(sqrtn));
        y <- tabulate(round(a*trace2+b),nbins=round(sqrtn));

        # Take x as the relative frequencies of the "theoretical" distribution and
        # adjust it to the number of occurrencies in y
        fac <- sum(y)/sum(x);
        x <- round(x * fac);

        z <- mergevec(x,y);

	# perform test
	chisqres <- chisq.test(z);
        # cat("chisqres: ", chisqres$p.value, " ",chisqres$statistic, "\n");
	# return result
	chisqres
}

#################################

# read command line arguments
args <- commandArgs(TRUE)
if (length(args) < 3) {
	stop("Missing command line parameters.")
}
infile1 <- args[1];
infile2 <- args[2];
outfile <- args[3];
# load traces
trace1 <- loadTrace(infile1);
trace2 <- loadTrace(infile2);
# perform tests
res1 <- testKS(trace1, trace2);
# res2 <- testT(trace1, trace2);
res3 <- testChisq(trace1, trace2);
# append ".tex" to output filename, since this is the
# convention for Tex-Files in a ProFiDo workflow
outfile <- paste(outfile,".tex",sep="");
# save test results in LaTeX format
# saveRes(outfile,res1,res2,res3);
saveRes(outfile,res1,res3);
q()

#################################

