#!/bin/bash

# This script performs a series of diff commands on the files within the 
# folders passed to it. Each file within the reference folder is compared
# to all files in the result folder which correspond to it. 
# It assumes that the files in the reference folder occur in pairs of the 
# form X.sh and X_setup.sh and structures the diff commands accordingly. 
# The file X.sh will be compared to X_1.sh, ..., X_m.sh, additionally the
# file X_setup.sh will be compared to X_1_setup.sh, ..., X_m_setup.sh.
# After identifying all pairs of reference files, the script sorts the 
# files within the result folder in subfolders representing the corresponding 
# pair of reference scripts. Afterwards the scripts in the result folder 
# are compared to the corresponding reference scripts.
# This script outputs a short overview containing a list of all exports 
# that did not match the corresponding reference file. Furthermore a 
# detailed report file for each reference file is generated in the 
# specified target folder, containing any differences to the corresponding 
# exports. In order to filter differences that will only result in a different
# order of execution line-wise alphabetical sorted versions of the input 
# files are com- pared additionally.
# Furthermore lines only containing irrelevant informations (Information about 
# the ProFiDo version the script was created with, user specified folders, etc.) 
# are filtered before any other processing using grep.
# This script is called the following way:
# 
# 	perform_diff.sh <reference folder> <result folder> <logfile>

# Check parameter count
EXPECTED_ARGS=3
if [ $# -ne $EXPECTED_ARGS ]
then
  echo -e "ProFiDo TestManager"
  echo -e "Usage: `basename $0` <reference folder> <result folder> <logfile>"
  echo -e ""
  echo -e "\t<reference folder>\tSpecifies the location where the" 
  echo -e "\t\t\t\treference exports are located. Reference"
  echo -e "\t\t\t\texports are expected to occur in pairs of"
  echo -e "\t\t\t\tthe form X.sh and X_setup.sh."
  echo -e ""
  echo -e "\t<result folder>\t\tSpecifies the location where the exports that" 
  echo -e "\t\t\t\tshould be tested are located."
  echo -e ""
  echo -e "\t<logfile>\t\tDesired destination of a brief logfile" 
  echo -e "\t\t\t\tproviding an overview which files differed"
  echo -e "\t\t\t\tfrom the cooresponding reference files."
  
  exit $E_BADARGS
fi


# Read parameters
REFERENCEFOLDER=$1
RESULTFOLDER=$2
LOGFILE=$3
#reportFileName=${RESULTFOLDER}/test_overview.txt

diffEgrepFilter="# This workflow was created with ProFiDo|export BASEDIR=|export TARGETDIR="


# This function performs the actual diff
function performDiff() {
	refFile=`basename "$1"`
	testedFile=`basename "$2"`
	echo $testedFile
	targetDir=$3

	# Prepare files
	egrep -v "$diffEgrepFilter" "$1">"fristFileGrep.tmp"
	egrep -v "$diffEgrepFilter" "$2">"secondFileGrep.tmp"
	sort -i fristFileGrep.tmp>"fristFileGrepSorted.tmp"
	sort -i secondFileGrep.tmp>"secondFileGrepSorted.tmp"

	# Performing diff
	diffResult=`diff -b fristFileGrep.tmp secondFileGrep.tmp`
	diffReportFileName=${targetDir}/${refFile}_diff.txt
	if [ "$diffResult" != "" ] ; then
		echo "=================================================================" >> "$diffReportFileName"
		echo "        diff $refFile $testedFile" >> "$diffReportFileName"
		echo "=================================================================" >> "$diffReportFileName"
		echo "$diffResult" >> "$diffReportFileName"
		echo "" >> "$diffReportFileName"
		diffDiffs="${diffDiffs}${refFile} ${testedFile}\n"		
	fi

	# Performing diff on sorted files
	#sortedDiffResult=`diff -b <(sort -i <(egrep -v "$diffEgrepFilter" "$1")) <(sort -i <(egrep -v "$diffEgrepFilter" "$2"))`
	sortedDiffResult=`diff -b fristFileGrepSorted.tmp secondFileGrepSorted.tmp`
	sortedDiffReportFileName=${targetDir}/${refFile}_sorted_diff.txt	
	if [ "$sortedDiffResult" != "" ] ; then
		echo "=================================================================" >> "$sortedDiffReportFileName"
		echo "     sorted diff $refFile $testedFile" >> "$sortedDiffReportFileName"
		echo "=================================================================" >> "$sortedDiffReportFileName"
		echo "$sortedDiffResult" >> "$sortedDiffReportFileName"
		echo "" >> "$sortedDiffReportFileName"
		sortedDiffDiffs="${sortedDiffDiffs}${refFile} ${testedFile}\n"
	fi
	
	rm *.tmp

}


# Iterate through all reference files and perform diff with corresponding exports
cd "$REFERENCEFOLDER"
echo "Comparing test results with reference files..." | tee -a $LOGFILE
for curRefFile in *setup.sh ; do
	# Should always occur in pairs (workflow and setup)	
	curRefFileBaseName="`basename $curRefFile _setup.sh`"
	
	# Create sub folders for all corresponding exports
	cd "$RESULTFOLDER"
	mkdir -p "$curRefFileBaseName/setup"
	mkdir -p "$curRefFileBaseName/workflow"

	# Sort exports into subfolders
	mv "${curRefFileBaseName}"*_setup.sh "${RESULTFOLDER}/${curRefFileBaseName}/setup/"
	mv "${curRefFileBaseName}"*.sh "${RESULTFOLDER}/${curRefFileBaseName}/workflow/"
	
	# Remove old report files
	rm -f "${RESULTFOLDER}/${curRefFile}.txt"
	rm -f "${RESULTFOLDER}/${curRefFile}_sorted_diff.txt"
	rm -f "${RESULTFOLDER}/${curRefFileBaseName}.sh.txt"
	rm -f "${RESULTFOLDER}/${curRefFileBaseName}.sh_sorted_diff.txt"

	# Perform actual diff for setup scripts
	cd "${RESULTFOLDER}/${curRefFileBaseName}/setup/"
	for curSetupFile in *.sh ; do
		performDiff "${REFERENCEFOLDER}/${curRefFile}" "${RESULTFOLDER}/${curRefFileBaseName}/setup/${curSetupFile}" ${RESULTFOLDER}
	done
	
	# Perform actual diff for workflow scripts
	cd "${RESULTFOLDER}/${curRefFileBaseName}/workflow/"
	for curWorkflowFile in *.sh ; do
		performDiff "${REFERENCEFOLDER}/${curRefFileBaseName}.sh" "${RESULTFOLDER}/${curRefFileBaseName}/workflow/${curWorkflowFile}" ${RESULTFOLDER}
	done	
done

# Print out report

if [ "$sortedDiffDiffs" != "" ] ; then
	echo "The following files didn't match after sorting:" | tee -a $LOGFILE
	echo -e "$sortedDiffDiffs" | tee -a $LOGFILE
else
	echo "All files matched after sorting." | tee -a $LOGFILE
fi

if [ "$diffDiffs" != "" ] ; then
	echo "The following original files didn't match:" | tee -a $LOGFILE
	echo -e "$diffDiffs" | tee -a $LOGFILE
else
	echo "All original files matched." | tee -a $LOGFILE
fi


