#!/bin/bash

# This script performs a series of tests on an executable jar version of ProFiDo. 
# When executing the script all workflows located at a specified location are 
# loaded and several test rounds are performed with every workflow, each consisting
# of loading, randomly modifying and exporting the workflow using the TestManager 
# described above. During the test rounds the exported workflow scripts are 
# continuously numbered allowing easy attribution of input workflows to the 
# corresponding export and test round. For a workflow located at the specified location 
# and named x.sh the created scripts will be named x_1.sh, ..., x_m.sh, where m 
# is the number of desired test rounds. (As ProFiDo creates a setup script for 
# each exported workflow script, additional files named x_1_setup.sh, ..., 
# x_m_setup.sh will be created.)
# In order to detect potential errors, the exported scripts are then compared to 
# reference exports of the corresponding workflow. For each workflow located 
# at the specified workflow folder, a reference export with the same base name 
# is expected to be located at the specified reference location. (Beside the 
# reference workflow script a corresponding reference setup script is expected
# for each workflow at the specified reference location.)
# The actual comparison is done by the provided perform_diffs.sh script.
# Further documentation of the perform_diffs.sh script is provided within the 
# comments in the first lines of the scripts file.
#
# Be aware that the provided scripts can be substantial help in detecting errors, 
# but should never be relied on exclusively, as only certain types of errors can 
# be detected this way.
#
# This script takes a series of parameters to specify the files and folders on 
# which it should operate and the desired amount of tests.
#
#	perform_tests.sh 	<profido jar> <workflow folder> <reference folder> 
#						<target folder> <test count> <actions per test>
#
#	<profido jar> 		Specifies the location of the JAR-File on which the tests 
#						should be performed.
#	<workflow folder> 	Defines the location where the workflows used for the tests 
#						are located. Any xml-file located in this folder will be used 
#						for the test.
#	<reference folder> 	Specifies the location where the reference exports of the 
#						workflows are located. A reference export for each workflow in 
#						the <workflow folder> is expected to be located here.
#	<target folder> 	Defines the folder where the exported workflows and report 
#						files will be stored.
#	<test count> 		Defines the number of test rounds that should be done on each 
#						workflow file.
#	<actions per test> 	Specifies the amount of user interactions (amount of deleted 
#						nodes and edges as described above) that should be done within 
#						one test round.
#
# Assuming a folder named /ProFiDo/testsuite/export_samples containing input 
# workflows and the corresponding reference exports, a sample call performing 
# 4 test rounds on each of the workflows with 8 user interactions per test round 
# would be of the following form:
#	./perform_tests.sh ~/ProFiDo/profido.jar ~/ProFiDo/testsuite/export_samples
#					   ~/ProFiDo/testsuite/export_samples ~/ProFiDo 4 8


# Check parameter count
EXPECTED_ARGS=6
if [ $# -ne $EXPECTED_ARGS ]
then
  echo -e "ProFiDo TestManager"
  echo -e "Usage:\t\t`basename $0` <profido jar> <workflow folder>"
  echo -e "\t\t\t\t <reference folder> <target folder> <test count>"
  echo -e "\t\t\t\t <actions per test>"  
  echo -e ""
  echo -e "Example:\t`basename $0` ~/ProFiDo/profido.jar"
  echo -e "\t\t\t\t ~/ProFiDo/testsuite/export_samples"
  echo -e "\t\t\t\t ~/ProFiDo/testsuite/export_samples"
  echo -e "\t\t\t\t ~/ProFiDo 4 8" 
  echo -e ""

  echo -e ""
  echo -e "\t<profido jar>\t\tSpecifies the location of the JAR-File on which"
  echo -e "\t\t\t\tthe tests should be performed."
  echo -e ""
  echo -e "\t<workflow folder>\tDefines the location where the workflows used"
  echo -e "\t\t\t\tfor the tests are located. All xml-files"
  echo -e "\t\t\t\tlocated in this folder will be used."
  echo -e ""
  echo -e "\t<reference folder>\tSpecifies the location where the reference" 
  echo -e "\t\t\t\texports of the workflows are located. A"
  echo -e "\t\t\t\treference export for each workflow in"
  echo -e "\t\t\t\tthe <workflow folder> is expected to be"
  echo -e "\t\t\t\tlocated here."
  echo -e ""
  echo -e "\t<target folder>\t\tDefines the folder where the exported"
  echo -e "\t\t\t\tworkflows and report files will be stored."	
  echo -e ""
  echo -e "\t<test count>\t\tDefines the number of test rounds that"
  echo -e "\t\t\t\tshould be done on each workflow file."
  echo -e ""
  echo -e "\t<actions per test>\tSpecifies the amount of user interactions"
  echo -e "\t\t\t\t(amount of deleted nodes and edges as described" 
  echo -e "\t\t\t\tabove) that should be performed within"
  echo -e "\t\t\t\tone test round."
  exit $E_BADARGS
fi

# Load command line parameters
PROFIDOJAR=$1
WORKFLOWFOLDER=$2
REFERENCEFOLDER=$3
TARGETFOLDER=$4
TESTCOUNT=$5
ACTIONSPERTEST=$6

# Determine path
rootDirOfScript="`cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd`"
jarDir="`dirname $PROFIDOJAR`"

# Create logfile
logFile=${TARGETFOLDER}/logfile.txt
echo "Logfile from perform_tests.sh" > "$logFile"
echo "" >> "$logFile"


# Iterate through all workflow files and create exports
cd "$WORKFLOWFOLDER"
for curWorkflow in *.xml ; do	
	# Start test for current workflow
	curWorkflowWithoutExtension="`basename $curWorkflow .xml`"
		
	# Perform specified number of rounds
	let curTestRound=1 
	while [ $curTestRound -le $TESTCOUNT ]; do
		curInputFile="${WORKFLOWFOLDER}/${curWorkflow}"
		curOutputFile="${TARGETFOLDER}/${curWorkflowWithoutExtension}_${curTestRound}.sh"
		cd "$jarDir"
		java -jar "${PROFIDOJAR}" "${curInputFile}" "${curOutputFile}" -t workflowPanel ${ACTIONSPERTEST} -d 10	-n | tee -a "$logFile"
		
		let "curTestRound = curTestRound + 1"
	done
done

# Perform diffs on created exports
cd "$rootDirOfScript"
./perform_diffs.sh "$REFERENCEFOLDER" "$TARGETFOLDER" "$logFile"







